@@ -106,7 +106,7 @@ def general_cosine(M, a, sym=True):
     >>> plt.ylabel("Normalized magnitude [dB]")
     >>> plt.xlabel("Normalized frequency [cycles per sample]")
     >>> plt.axhline(-90.2, color='red')
-
+    >>> plt.show()
     """
     if _len_guards(M):
         return np.ones(M)
@@ -664,7 +664,7 @@ def bartlett(M, sym=True):
     .. [3] A.V. Oppenheim and R.W. Schafer, "Discrete-Time Signal
            Processing", Prentice-Hall, 1999, pp. 468-471.
     .. [4] Wikipedia, "Window function",
-           http://en.wikipedia.org/wiki/Window_function
+           https://en.wikipedia.org/wiki/Window_function
     .. [5] W.H. Press,  B.P. Flannery, S.A. Teukolsky, and W.T. Vetterling,
            "Numerical Recipes", Cambridge University Press, 1986, page 429.
 
@@ -753,7 +753,7 @@ def hann(M, sym=True):
     .. [2] E.R. Kanasewich, "Time Sequence Analysis in Geophysics",
            The University of Alberta Press, 1975, pp. 106-108.
     .. [3] Wikipedia, "Window function",
-           http://en.wikipedia.org/wiki/Window_function
+           https://en.wikipedia.org/wiki/Window_function
     .. [4] W.H. Press,  B.P. Flannery, S.A. Teukolsky, and W.T. Vetterling,
            "Numerical Recipes", Cambridge University Press, 1986, page 425.
 
@@ -785,6 +785,7 @@ def hann(M, sym=True):
     # Docstring adapted from NumPy's hanning function
     return general_hamming(M, 0.5, sym)
 
+
 @np.deprecate(new_name='scipy.signal.windows.hann')
 def hanning(*args, **kwargs):
     return hann(*args, **kwargs)
@@ -820,7 +821,7 @@ def tukey(M, alpha=0.5, sym=True):
            Analysis with the Discrete Fourier Transform". Proceedings of the
            IEEE 66 (1): 51-83. :doi:`10.1109/PROC.1978.10837`
     .. [2] Wikipedia, "Window function",
-           http://en.wikipedia.org/wiki/Window_function#Tukey_window
+           https://en.wikipedia.org/wiki/Window_function#Tukey_window
 
     Examples
     --------
@@ -927,6 +928,7 @@ def barthann(M, sym=True):
 
     return _truncate(w, needs_trunc)
 
+
 def general_hamming(M, alpha, sym=True):
     r"""Return a generalized Hamming window.
 
@@ -970,26 +972,24 @@ def general_hamming(M, alpha, sym=True):
     --------
     The Sentinel-1A/B Instrument Processing Facility uses generalized Hamming
     windows in the processing of spaceborne Synthetic Aperture Radar (SAR)
-    data [3]_. The facility uses various values for the :math:`\alpha` parameter
-    based on operating mode of the SAR instrument. Some common :math:`\alpha`
-    values include 0.75, 0.7 and 0.52 [4]_. As an example, we plot these different
-    windows.
+    data [3]_. The facility uses various values for the :math:`\alpha`
+    parameter based on operating mode of the SAR instrument. Some common
+    :math:`\alpha` values include 0.75, 0.7 and 0.52 [4]_. As an example, we
+    plot these different windows.
 
     >>> from scipy.signal.windows import general_hamming
     >>> from scipy.fftpack import fft, fftshift
     >>> import matplotlib.pyplot as plt
 
-    >>> plt.figure()
-    >>> plt.title("Generalized Hamming Windows")
-    >>> plt.ylabel("Amplitude")
-    >>> plt.xlabel("Sample")
-    >>> spatial_plot = plt.axes()
+    >>> fig1, spatial_plot = plt.subplots()
+    >>> spatial_plot.set_title("Generalized Hamming Windows")
+    >>> spatial_plot.set_ylabel("Amplitude")
+    >>> spatial_plot.set_xlabel("Sample")
 
-    >>> plt.figure()
-    >>> plt.title("Frequency Responses")
-    >>> plt.ylabel("Normalized magnitude [dB]")
-    >>> plt.xlabel("Normalized frequency [cycles per sample]")
-    >>> freq_plot = plt.axes()
+    >>> fig2, freq_plot = plt.subplots()
+    >>> freq_plot.set_title("Frequency Responses")
+    >>> freq_plot.set_ylabel("Normalized magnitude [dB]")
+    >>> freq_plot.set_xlabel("Normalized frequency [cycles per sample]")
 
     >>> for alpha in [0.75, 0.7, 0.52]:
     ...     window = general_hamming(41, alpha)
@@ -1006,7 +1006,7 @@ def general_hamming(M, alpha, sym=True):
     .. [1] DSPRelated, "Generalized Hamming Window Family",
            https://www.dsprelated.com/freebooks/sasp/Generalized_Hamming_Window_Family.html
     .. [2] Wikipedia, "Window function",
-           http://en.wikipedia.org/wiki/Window_function
+           https://en.wikipedia.org/wiki/Window_function
     .. [3] Riccardo Piantanida ESA, "Sentinel-1 Level 1 Detailed Algorithm
            Definition",
            https://sentinel.esa.int/documents/247904/1877131/Sentinel-1-Level-1-Detailed-Algorithm-Definition
@@ -1061,7 +1061,7 @@ def hamming(M, sym=True):
     .. [2] E.R. Kanasewich, "Time Sequence Analysis in Geophysics", The
            University of Alberta Press, 1975, pp. 109-110.
     .. [3] Wikipedia, "Window function",
-           http://en.wikipedia.org/wiki/Window_function
+           https://en.wikipedia.org/wiki/Window_function
     .. [4] W.H. Press,  B.P. Flannery, S.A. Teukolsky, and W.T. Vetterling,
            "Numerical Recipes", Cambridge University Press, 1986, page 425.
 
@@ -1093,6 +1093,7 @@ def hamming(M, sym=True):
     # Docstring adapted from NumPy's hamming function
     return general_hamming(M, 0.54, sym)
 
+
 def kaiser(M, beta, sym=True):
     r"""Return a Kaiser window.
 
@@ -1168,7 +1169,7 @@ def kaiser(M, beta, sym=True):
     .. [2] E.R. Kanasewich, "Time Sequence Analysis in Geophysics", The
            University of Alberta Press, 1975, pp. 177-178.
     .. [3] Wikipedia, "Window function",
-           http://en.wikipedia.org/wiki/Window_function
+           https://en.wikipedia.org/wiki/Window_function
     .. [4] F. J. Harris, "On the use of windows for harmonic analysis with the
            discrete Fourier transform," Proceedings of the IEEE, vol. 66,
            no. 1, pp. 51-83, Jan. 1978. :doi:`10.1109/PROC.1978.10837`.
@@ -1709,7 +1710,7 @@ def dpss(M, NW, Kmax=None, sym=True, norm=None, return_ratios=False):
     """
     Compute the Discrete Prolate Spheroidal Sequences (DPSS).
 
-    DPSS (or Slepian sequencies) are often used in multitaper power spectral
+    DPSS (or Slepian sequences) are often used in multitaper power spectral
     density estimation (see [1]_). The first window in the sequence can be
     used to maximize the energy concentration in the main lobe, and is also
     called the Slepian window.
@@ -1902,7 +1903,7 @@ def dpss(M, NW, Kmax=None, sym=True, norm=None, return_ratios=False):
     # sequences, or discrete prolate spheroidal sequences (DPSS). Only the
     # first K, K = 2NW/dt orders of DPSS will exhibit good spectral
     # concentration
-    # [see http://en.wikipedia.org/wiki/Spectral_concentration_problem]
+    # [see https://en.wikipedia.org/wiki/Spectral_concentration_problem]
 
     # Here we set up an alternative symmetric tri-diagonal eigenvalue
     # problem such that
